@@ -11,18 +11,18 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 // You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
- 
+
 // (c) COPYRIGHT URI/MIT 1994-1996
 // Please read the full copyright statement in the file COPYRIGHT.
 //
@@ -31,18 +31,21 @@
 
 // DODS-netCDF surrogate library functions for data attributes. See note
 // about the symbol LOCAL in Dfile.c
-// 
+//
 // ReZa 10/4/94
 
 #include "config_nc.h"
 
 #include <cstring>
 
-static char rcsid[] not_used ={"$Id: Dattr.cc 18782 2008-05-23 16:39:07Z jimg $"};
+static char rcsid[] not_used ={"$Id: Dattr.cc 19366 2008-09-08 19:26:40Z jimg $"};
 
 #include "Dnetcdf.h"
 #include "Dncx.h"
 
+#include "Connections.h"
+#include "Connections.cc"
+
 #include "nc_util.h"
 
 /** Defined in lnetcdf/lerror. This is not included in a header (like
@@ -54,13 +57,13 @@ int
 nc_put_att_text(int cdfid, int varid, const char *name,
         size_t nelems, const char *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_text((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_text((*conns)[cdfid]->get_ncid(), varid, name,
 			  nelems, value);
 	return rcode;
     }
@@ -73,13 +76,13 @@ int
 nc_put_att_schar(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const signed char *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_schar((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_schar((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -91,14 +94,14 @@ nc_put_att_schar(int cdfid, int varid, const char *name,
 int
 nc_put_att_uchar(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const unsigned char *value)
-{ 
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+{
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_uchar((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_uchar((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -106,18 +109,18 @@ nc_put_att_uchar(int cdfid, int varid, const char *name,
 
     return NC_EPERM; // DODS network interface is read-only
 }
- 
+
 int
 nc_put_att_short(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const short *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_short((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_short((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -130,13 +133,13 @@ int
 nc_put_att_int(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const int *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_int((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_int((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -149,13 +152,13 @@ int
 nc_put_att_long(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const long *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_long((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_long((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -169,13 +172,13 @@ int
 nc_put_att_float(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const float *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_float((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_float((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -188,13 +191,13 @@ int
 nc_put_att_double(int cdfid, int varid, const char *name,
         nc_type type, size_t nelems, const double *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att_double((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att_double((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
@@ -203,31 +206,31 @@ nc_put_att_double(int cdfid, int varid, const char *name,
     return NC_EPERM; // DODS network interface is read-only
 }
 
-// Used for the netCDF 2 interface emulation. See lv2i.c. 
+// Used for the netCDF 2 interface emulation. See lv2i.c.
 extern "C" int
 nc_put_att(int cdfid, int varid, const char *name, nc_type type,
         size_t nelems, const void *value)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_put_att((*conns)[cdfid]->get_ncid(), varid, name, 
+	rcode = lnc_put_att((*conns)[cdfid]->get_ncid(), varid, name,
 			  type, nelems, value);
 	return rcode;
     }
 #endif
 
     return NC_EPERM; // DODS network interface is read-only
-} 
+}
 
 
 int
 nc_rename_att(int cdfid, int varid, const char *name, const char *newname)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -244,7 +247,7 @@ nc_rename_att(int cdfid, int varid, const char *name, const char *newname)
 int
 nc_del_att(int cdfid, int varid, const char *name)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -264,7 +267,7 @@ nc_inq_attname(int cdfid, int varid, int attnum, char *name)
 #if 0
     NCConnect &ncc = *((*conns)[cdfid]);
 #endif
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -294,14 +297,14 @@ nc_inq_attname(int cdfid, int varid, int attnum, char *name)
     return NC_NOERR;
 }
 
-int 
+int
 nc_inq_attid(int cdfid, int varid, const char *name, int *attnump)
 {
 #if 0
     NCConnect &ncc = *((*conns)[cdfid]);
 #endif
 
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -315,7 +318,7 @@ nc_inq_attid(int cdfid, int varid, const char *name, int *attnump)
     try {
 	AttrTable &attr = (*conns)[cdfid]->get_attribute_table(varid);
 	AttrTable::Attr_iter p = attr.attr_begin();
- 
+
 	for (int i = 0; p != attr.attr_end(); ++p, ++i) {
 	    if (attr.get_name(p) == name) {
 		if (attnump != 0)
@@ -328,21 +331,21 @@ nc_inq_attid(int cdfid, int varid, const char *name, int *attnump)
         set_opendap_error(e);
 	return e.get_error_code();
     }
-    
+
     return NC_ENOTATT;
 }
 
 int
-nc_inq_att(int cdfid, int varid, const char *name, 
+nc_inq_att(int cdfid, int varid, const char *name,
 	   nc_type *datatypep, size_t *lenp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_inq_att( (*conns)[cdfid]->get_ncid(), varid, 
+	rcode = lnc_inq_att( (*conns)[cdfid]->get_ncid(), varid,
 			   name, datatypep, lenp);
 	return rcode;
     }
@@ -370,7 +373,7 @@ nc_inq_att(int cdfid, int varid, const char *name,
 	DBG(cerr << "After attr.get_attr(p)" << endl);
 	AttrType attrT = attr.get_attr_type(p);
 
-	if (lenp) 
+	if (lenp)
             *lenp = static_cast<size_t>(attr.get_attr_num(p));
 
 	switch (attrT) {
@@ -414,16 +417,16 @@ nc_inq_att(int cdfid, int varid, const char *name,
     return NC_NOERR ;
 }
 
-int 
+int
 nc_inq_attlen(int cdfid, int varid, const char *name, size_t *lenp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_inq_attlen( (*conns)[cdfid]->get_ncid(), varid, 
+	rcode = lnc_inq_attlen( (*conns)[cdfid]->get_ncid(), varid,
 			   name, lenp);
 	return rcode;
     }
@@ -463,19 +466,19 @@ nc_inq_attlen(int cdfid, int varid, const char *name, size_t *lenp)
     return NC_NOERR;
 }
 
-int 
+int
 nc_inq_atttype(int cdfid, int varid, const char *name, nc_type *datatypep)
 {
 #if 0
     NCConnect &ncc = *((*conns)[cdfid]);
 #endif
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[cdfid]->is_local()) {
-	rcode = lnc_inq_atttype( (*conns)[cdfid]->get_ncid(), varid, 
+	rcode = lnc_inq_atttype( (*conns)[cdfid]->get_ncid(), varid,
 			   name, datatypep);
 	return rcode;
     }
@@ -533,16 +536,16 @@ int
 nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
 {
     // does the file id exist?
-    if (!conns || incdf < 0 || incdf > MAX_NC_OPEN 
-	|| outcdf < 0 || outcdf > MAX_NC_OPEN 
-	|| (*conns)[incdf] == NULL || (*conns)[outcdf] == NULL) 
+    if (!conns || incdf < 0 || incdf > MAX_NC_OPEN
+	|| outcdf < 0 || outcdf > MAX_NC_OPEN
+	|| (*conns)[incdf] == NULL || (*conns)[outcdf] == NULL)
 	return NC_EBADID;
 
     // case 1, both input and output files are local
 
 #ifdef LOCAL
     if ((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
-        rcode = lnc_copy_att((*conns)[incdf]->get_ncid(), invar, 
+        rcode = lnc_copy_att((*conns)[incdf]->get_ncid(), invar,
                     name, (*conns)[outcdf]->get_ncid(), outvar);
 	    return rcode;
     }
@@ -550,14 +553,14 @@ nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
 
     // case 2, the output file is not local, Error writing over the net.
 
-    if (!(*conns)[outcdf]->is_local()) 
+    if (!(*conns)[outcdf]->is_local())
       return NC_EPERM; // DODS network interface is read-only
 
     // last case, input is from DODS file and output a local netcdf file
 
 #ifdef LOCAL
     char *values = 0;         // Attribute vector to be copied,
-    nc_type datatype;    // it's type, 
+    nc_type datatype;    // it's type,
     size_t count;          // and it's length.
 
     try {
@@ -573,7 +576,7 @@ nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
     // real) netCDF library. When calling our functions, make sure to pass
     // the index to conns. jhrg 6/1/05
 #if 0
-    rcode = nc_put_att((*conns)[outcdf]->get_ncid(), outvar,  
+    rcode = nc_put_att((*conns)[outcdf]->get_ncid(), outvar,
 		       name, datatype, count, (void *)values);
 #endif
     DBG(cerr << "nc_copy_att" << endl);
@@ -592,7 +595,7 @@ nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
 int
 nc_get_att_text(int cdfid, int varid, const char *name, char *str)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -634,7 +637,7 @@ nc_get_att_text(int cdfid, int varid, const char *name, char *str)
 int
 nc_get_att_schar(int cdfid, int varid, const char *name, signed char *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -669,7 +672,7 @@ nc_get_att_schar(int cdfid, int varid, const char *name, signed char *tp)
 int
 nc_get_att_uchar(int cdfid, int varid, const char *name, unsigned char *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -690,7 +693,7 @@ nc_get_att_uchar(int cdfid, int varid, const char *name, unsigned char *tp)
 	    // return NC_ECHAR;
 
 	rcode = putn_into_uchar((void **)&values, count, tp, dt);
-    
+
 	delete [] values; values = 0;
 	return rcode;
     }
@@ -704,7 +707,7 @@ nc_get_att_uchar(int cdfid, int varid, const char *name, unsigned char *tp)
 int
 nc_get_att_short(int cdfid, int varid, const char *name, short *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -738,7 +741,7 @@ nc_get_att_short(int cdfid, int varid, const char *name, short *tp)
 int
 nc_get_att_int(int cdfid, int varid, const char *name, int *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -772,7 +775,7 @@ nc_get_att_int(int cdfid, int varid, const char *name, int *tp)
 int
 nc_get_att_long(int cdfid, int varid, const char *name, long *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -806,7 +809,7 @@ nc_get_att_long(int cdfid, int varid, const char *name, long *tp)
 int
 nc_get_att_float(int cdfid, int varid, const char *name, float *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -840,7 +843,7 @@ nc_get_att_float(int cdfid, int varid, const char *name, float *tp)
 int
 nc_get_att_double(int cdfid, int varid, const char *name, double *tp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -871,11 +874,11 @@ nc_get_att_double(int cdfid, int varid, const char *name, double *tp)
     }
 }
 
-// Used for the netCDF 2 interface emulation. See lv2i.c. 
+// Used for the netCDF 2 interface emulation. See lv2i.c.
 extern "C" int
 nc_get_att(int cdfid, int varid, const char *name, void *values)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
