@@ -12,12 +12,12 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -30,19 +30,22 @@
 // Authors:
 //      reza            Reza Nekovei (reza@intcomm.net)
 
-// DODS-netCDF surrogate library for data variables. 
-// 
+// DODS-netCDF surrogate library for data variables.
+//
 // ReZa 10/6/94
 
 #include "config_nc.h"
 
 #include <cstring>
 
-static char rcsid[] not_used ={"$Id: Dvar.cc 18782 2008-05-23 16:39:07Z jimg $"};
+static char rcsid[] not_used ={"$Id: Dvar.cc 19366 2008-09-08 19:26:40Z jimg $"};
 
 #include "Dnetcdf.h"
 #include "nc_util.h"
 
+#include "Connections.h"
+#include "Connections.cc"
+
 #include <BaseType.h>
 
 /** Defined in lnetcdf/lerror. This is not included in a header (like
@@ -54,7 +57,7 @@ int
 nc_def_var( int cdfid, const char *name, nc_type type,
          int ndims, const int *dimids, int *varidp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -72,7 +75,7 @@ nc_def_var( int cdfid, const char *name, nc_type type,
 int
 nc_inq_varid(int cdfid, const char *name, int *varid_ptr)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -100,7 +103,7 @@ nc_inq_varid(int cdfid, const char *name, int *varid_ptr)
 int
 nc_rename_var(int cdfid, int varid, const char *newname)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -118,7 +121,7 @@ int
 nc_inq_var(int cdfid, int varid, char *name, nc_type *typep,
         int *ndimsp, int *dimids, int *nattsp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -141,7 +144,7 @@ nc_inq_var(int cdfid, int varid, char *name, nc_type *typep,
 
 	if (nattsp != 0)
 	    *nattsp = (*conns)[cdfid]->get_num_attr(varid);
-    
+
     }
     catch (Error &e) {
         set_opendap_error(e);
@@ -152,10 +155,10 @@ nc_inq_var(int cdfid, int varid, char *name, nc_type *typep,
 }
 
 
-int 
+int
 nc_inq_varname(int cdfid, int varid, char *name)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -181,10 +184,10 @@ nc_inq_varname(int cdfid, int varid, char *name)
 }
 
 
-int 
+int
 nc_inq_vartype(int cdfid, int varid, nc_type *typep)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -206,14 +209,14 @@ nc_inq_vartype(int cdfid, int varid, nc_type *typep)
 	return e.get_error_code();
     }
 
-    return NC_NOERR; 
+    return NC_NOERR;
 }
 
 
-int 
+int
 nc_inq_varndims(int cdfid, int varid, int *ndimsp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -238,14 +241,14 @@ nc_inq_varndims(int cdfid, int varid, int *ndimsp)
 	return e.get_error_code();
     }
 
-    return NC_NOERR; 
+    return NC_NOERR;
 }
 
 
-int 
+int
 nc_inq_vardimid(int cdfid, int varid, int *dimids)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -271,10 +274,10 @@ nc_inq_vardimid(int cdfid, int varid, int *dimids)
 }
 
 
-int 
+int
 nc_inq_varnatts(int cdfid, int varid, int *nattsp)
 {
-    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
+    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN
 	|| (*conns)[cdfid] == NULL) // does the file id exist?
 	return NC_EBADID;
 
@@ -290,22 +293,22 @@ nc_inq_varnatts(int cdfid, int varid, int *nattsp)
       return NC_ENOTVAR;
 
     if(nattsp != 0)         // Gets NC_GLOBAL when varid is -1
-      *nattsp = (*conns)[cdfid]->get_num_attr(varid); 
-    
+      *nattsp = (*conns)[cdfid]->get_num_attr(varid);
+
     return NC_NOERR;
 }
 
 int
 nc_copy_var(int ncid_in, int varid, int ncid_out)
 {
-    if (!conns || ncid_in < 0 || ncid_in > MAX_NC_OPEN 
-	|| ncid_out < 0 || ncid_out > MAX_NC_OPEN 
-	|| (*conns)[ncid_in] == NULL || (*conns)[ncid_out] == NULL) 
+    if (!conns || ncid_in < 0 || ncid_in > MAX_NC_OPEN
+	|| ncid_out < 0 || ncid_out > MAX_NC_OPEN
+	|| (*conns)[ncid_in] == NULL || (*conns)[ncid_out] == NULL)
 	return NC_EBADID;
 
 #ifdef LOCAL
     if((*conns)[ncid_in]->is_local() && (*conns)[ncid_out]->is_local()) {
-	rcode = lnc_copy_var((*conns)[ncid_in]->get_ncid(), varid, 
+	rcode = lnc_copy_var((*conns)[ncid_in]->get_ncid(), varid,
 			  (*conns)[ncid_out]->get_ncid());
 	return rcode;
     }
@@ -352,7 +355,7 @@ nc_copy_var(int ncid_in, int varid, int ncid_out)
 	int outdim_id;
 	status = nc_inq_dimid(ncid_out, dname, &outdim_id);
 	if (status != NC_NOERR) {
-	    nc_redef(ncid_out); 
+	    nc_redef(ncid_out);
 	    nc_def_dim(ncid_out, dname, size, &outdims[i]);
 	    nc_enddef(ncid_out);
 	}
@@ -368,7 +371,7 @@ nc_copy_var(int ncid_in, int varid, int ncid_out)
     }
 
     //create the new variable
-    nc_redef(ncid_out); 
+    nc_redef(ncid_out);
     int outvar;
     status = nc_def_var(ncid_out, name, datatype, ndims, outdims, &outvar);
     if (status != NC_NOERR)
