@@ -39,22 +39,18 @@
 #include "config_nc.h"
 
 static char rcsid[] not_used =
-    { "$Id: NCArray.cc 16556 2007-05-23 21:13:45Z jimg $"
+    { "$Id: NCArray.cc 18782 2008-05-23 16:39:07Z jimg $"
     };
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
-
+#include <cstring>
 #include <iostream>
 #include <sstream>
 #include <algorithm>
 
 // #define DODS_DEBUG 1
 
-#include "Error.h"
-#include "InternalErr.h"
+#include <Error.h>
+#include <InternalErr.h>
 
 #include "Dnetcdf.h"
 #include "Dncx.h"
@@ -332,7 +328,6 @@ void NCArray::extract_values(void *values, int /*elements*/, int outtype,
             buf2val((void **) spa);
 
             DBG(cerr << "nels: " << nels << endl);
-            DBG(cerr << "strdim: " << strdim << endl);
             DBG(for (int i = 0; i < nels; ++i) cerr << sa[i] << endl);
 
             for (int i = 0; i < nels; i++) {
