@@ -38,7 +38,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: NCSequence.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used ={"$Id: NCSequence.cc 13725 2006-04-19 18:06:01Z jimg $"};
 
 #include <sstream>
 #include <algorithm>
@@ -172,7 +172,7 @@ NCSequence::store_projection(const string &proj)
     even when the client asks for just one field. A bug in the Sequence CE
     processing code. */
 string
-NCSequence::build_constraint(int outtype, const size_t *start,
+NCSequence::build_constraint(int /*outtype*/, const size_t *start,
     const size_t *edges, const ptrdiff_t *stride) throw(Error)
 {
     string expr = name();      // CE always starts with the variable's name
@@ -348,9 +348,9 @@ NCSequence::flatten(const ClientParams &cp, const string &parent_name)
     while (field != field_end) {
         // Special case for embedded Sequences: not supported yet.
         // Really need to test for a sequence at any level... 4/8/05 jhrg
-        BaseType *ncs;                  // Really a NCSequence
+        BaseType *ncs = 0;                  // Really a NCSequence
         if ((*field)->type() == dods_sequence_c 
-            || (ncs = dynamic_cast<NCAccess*>(*field)->find_child_sequence())) {
+            || (ncs = dynamic_cast<NCAccess&>(**field).find_child_sequence())) {
             string attribute = "\"Elided inner Sequence named '";
             attribute += (ncs) ? ncs->name() : (*field)->name();
             attribute += "'\"";
