@@ -38,7 +38,7 @@
 
 #include "config_nc.h"
 
-static char rcsid[] not_used ={"$Id: NCStr.cc 11906 2005-08-08 19:51:43Z root $"};
+static char rcsid[] not_used ={"$Id: NCStr.cc 13725 2006-04-19 18:06:01Z jimg $"};
 
 // #define DODS_DEBUG 1
 
@@ -90,21 +90,23 @@ NCStr::ptr_duplicate()
 }
 
 void
-NCStr::extract_values(void *values, int elements, int outtype) throw(Error)
+NCStr::extract_values(void *values, int elements, int /*outtype*/,
+                      const ClientParams &cp) throw(Error)
 {
     // If this variable is held by a sequence, we need to treat it specially.
     // Read values from the Sequence and then use this instance to process
     // the values. 
     NCSequence *ncq = dynamic_cast<NCSequence*>(find_ancestral_sequence(this));
     int nels = (ncq) ? ncq->number_of_rows() : 1;
-
+    unsigned int dimsize=get_strdim();
+    
     DBG(cerr << "nels: " << nels << endl);
     
     char * tbfr = (char *)values;
     int i = 0;
     while (i < nels) {
-        string *cp = 0;
-        string **cpp = &cp;
+        string *sp = 0;
+        string **spp = &sp;
         // If the parent of this scalar is a Sequence, use that Sequence
         // to read values.
         if (ncq) {
@@ -112,57 +114,49 @@ NCStr::extract_values(void *values, int elements, int outtype) throw(Error)
             NCStr *s = dynamic_cast<NCStr*>(ncq->var_value(i, name()));
             if (!s)
                 throw InternalErr(__FILE__, __LINE__, "Bad csat to NCStr.");
-            s->buf2val((void **)cpp);
+            s->buf2val((void **)spp);
         }
         else {     
-            buf2val((void **)cpp);
+            buf2val((void **)spp);
         }
         
-#ifdef STRING_AS_ARRAY
-        if (get_translated()) {
+        if (cp.get_string_as_char_array() && get_translated()) {
             unsigned int c = 0;
-            while (c < STRING_ARRAY_SIZE 
-                   && (c < cp->length()
-                       || (cp->length() == 0 && c == 0))) {
-                *tbfr++ = *(cp->c_str() + c++);
+            while (c < dimsize
+                   && (c < sp->length()
+                       || (sp->length() == 0 && c == 0))) {
+                *tbfr++ = *(sp->c_str() + c++);
             }
-            while (c++ < STRING_ARRAY_SIZE) {
+            while (c++ < dimsize ) {
                 *tbfr++ = 0;
             }
         }
         else {
-#endif
             for (unsigned int c = 0;
-                 c < cp->length() || (cp->length() == 0 && c == 0);
+                 c < sp->length() || (sp->length() == 0 && c == 0);
                  c++) {
                  // I think it may be a bug to transfer more than one char since
                  // the client program has allocated a nels buffer for the data.
                  // jhrg 2/24/05
-                 *tbfr++ = *(cp->c_str() + c);
+                 *tbfr++ = *(sp->c_str() + c);
             }
-#ifdef STRING_AS_ARRAY
-        }       // Closes the 'else' above
-#endif
+        }
 
         ++i;
-        DBG(cerr << "Value: " << *cp << endl);
+        DBG(cerr << "Value: " << *sp << endl);
         // Now get rid of the C++ string object.
-        delete cp;
+        delete sp;
     }
     while (i++ < elements) {
-#ifdef STRING_AS_ARRAY
-        if (get_translated()) {
-            int c = 0;
-            while (c++ < STRING_ARRAY_SIZE) {
+        if (cp.get_string_as_char_array() && get_translated()) {
+            unsigned int c = 0;
+            while (c++ < dimsize) {
                 *tbfr++ = 0;
             }
         }
         else {
-#endif
             *tbfr++ = 0;
-#ifdef STRING_AS_ARRAY
         }
-#endif
     }
 }
 
