@@ -1,3 +1,15 @@
+/*
+This file is part of the netCDF Fortran 77 API.
+
+This file handles the netCDF strerror and libvers functions.
+
+Copyright 2006, University Corporation for Atmospheric Research. See
+the COPYRIGHT file for copying and redistribution conditions.
+
+$Id: fort-misc.c 15015 2006-11-03 17:39:01Z jimg $
+*/
+
+#include <config.h>
 #include "netcdf.h"
 #include "ncfortran.h"
 
