@@ -1,4 +1,4 @@
-/* $Id: ncfortran.h 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: ncfortran.h 15015 2006-11-03 17:39:01Z jimg $ */
 
 /*
  * This file contains `cfortran.h' definitions that are specific to the
@@ -9,7 +9,6 @@
 #ifndef UD_NCFORTRAN_H
 #define UD_NCFORTRAN_H
 
-
 #include "cfortran.h"
 #include "nfconfig.inc"
 
@@ -217,6 +216,11 @@
  */
 #define	PNATTS	PNDIMS
 
+/*
+ * Output format version number
+ */
+#define PFORMAT  PNDIMS
+
 
 /*******************************************************************************
  * netCDf variables:
@@ -473,7 +477,7 @@
 #define PDIMID_cfTYPE			NF_INTEGER
 #define PDIMID_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PDIMID,A,B,C,D,E)
 #define PDIMID_cfH(S,U,B)
-#define PDIMID_cfQ(B)			int B;
+#define PDIMID_cfQ(B)			int B = -1;
 #define PDIMID_cfT(M,I,A,B,D)		&B
 #define PDIMID_cfR(A,B,D)		*A=(B == -1 ? -1 : B+1);
 
@@ -485,7 +489,7 @@
 #define DIMIDSVVVVVVV_cfTYPE		NF_INTEGER
 #define DIMIDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,DIMIDS,A,B,C,D,E)
 #define DIMIDS_cfH(S,U,B)
-#define DIMIDS_cfQ(B)			int B[MAX_NC_DIMS];
+#define DIMIDS_cfQ(B)			int B[NC_MAX_DIMS];
 #define DIMIDS_cfT(M,I,A,B,D)		f2c_dimids(*fndims, A, B)
 #define DIMIDS_cfR(A,B,D)
 
@@ -497,7 +501,7 @@
 #define PDIMIDS_cfTYPE			NF_INTEGER
 #define PDIMIDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,PDIMIDS,A,B,C,D,E)
 #define PDIMIDS_cfH(S,U,B)
-#define PDIMIDS_cfQ(B)			int B[MAX_NC_DIMS];
+#define PDIMIDS_cfQ(B)			int B[NC_MAX_DIMS];
 #define PDIMIDS_cfT(M,I,A,B,D)		B
 #define PDIMIDS_cfR(A,B,D)		c2f_dimids(*fncid, *fvarid-1, B, A);
 
@@ -564,7 +568,7 @@
 #define COORDSVVVVVVV_cfTYPE		NF_INTEGER
 #define COORDS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,COORDS,A,B,C,D,E)
 #define COORDS_cfH(S,U,B)
-#define COORDS_cfQ(B)			size_t B[MAX_NC_DIMS];
+#define COORDS_cfQ(B)			size_t B[NC_MAX_DIMS];
 #define COORDS_cfT(M,I,A,B,D)		f2c_coords(*fncid, *fvarid-1, A, B)
 #define COORDS_cfR(A,B,D)
 
@@ -592,7 +596,7 @@
 #define COUNTSVVVVVVV_cfTYPE		NF_INTEGER
 #define COUNTS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,COUNTS,A,B,C,D,E)
 #define COUNTS_cfH(S,U,B)
-#define COUNTS_cfQ(B)			size_t B[MAX_NC_DIMS];
+#define COUNTS_cfQ(B)			size_t B[NC_MAX_DIMS];
 #define COUNTS_cfT(M,I,A,B,D)		f2c_counts(*fncid, *fvarid-1, A, B)
 #define COUNTS_cfR(A,B,D)
 
@@ -604,7 +608,7 @@
 #define STRIDESVVVVVVV_cfTYPE		NF_INTEGER
 #define STRIDES_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,STRIDES,A,B,C,D,E)
 #define STRIDES_cfH(S,U,B)
-#define STRIDES_cfQ(B)			ptrdiff_t B[MAX_NC_DIMS];
+#define STRIDES_cfQ(B)			ptrdiff_t B[NC_MAX_DIMS];
 #define STRIDES_cfT(M,I,A,B,D)		f2c_strides(*fncid, *fvarid-1, A, B)
 #define STRIDES_cfR(A,B,D)
 
@@ -616,7 +620,7 @@
 #define MAPSVVVVVVV_cfTYPE		NF_INTEGER
 #define MAPS_cfSTR(N,T,A,B,C,D,E)	_(CFARGS,N)(T,MAPS,A,B,C,D,E)
 #define MAPS_cfH(S,U,B)
-#define MAPS_cfQ(B)			ptrdiff_t B[MAX_NC_DIMS];
+#define MAPS_cfQ(B)			ptrdiff_t B[NC_MAX_DIMS];
 #define MAPS_cfT(M,I,A,B,D)		f2c_maps(*fncid, *fvarid-1, A, B)
 #define MAPS_cfR(A,B,D)
 
@@ -625,7 +629,7 @@
  * The following is for f2c-support only.
  */
 
-#if defined(f2cFortran)
+#if defined(f2cFortran) && !defined(pgiFortran)
 
 /*
  * The f2c(1) utility on BSD/OS and Linux systems adds an additional
@@ -641,6 +645,7 @@
 #define nf_create		nf_create_
 #define nf_open			nf_open_
 #define nf_set_fill		nf_set_fill_
+#define nf_set_default_format	nf_set_default_format_
 #define nf_redef		nf_redef_
 #define nf_enddef		nf_enddef_
 #define nf_sync			nf_sync_
@@ -652,6 +657,7 @@
 #define nf_inq_nvars		nf_inq_nvars_
 #define nf_inq_natts		nf_inq_natts_
 #define nf_inq_unlimdim		nf_inq_unlimdim_
+#define nf_inq_format		nf_inq_format_
 #define nf_def_dim		nf_def_dim_
 #define nf_inq_dimid		nf_inq_dimid_
 #define nf_inq_dim		nf_inq_dim_
