@@ -23,6 +23,8 @@
 
 #include "Error.h"              /* include for OPeNDAP error codes */
 
+using namespace libdap ;
+
 /* This is used to record the most recent opendap Error object. */
 static Error *opendap_error = 0;
 
@@ -125,7 +127,6 @@ static char unknown[] = "Unknown Error";
 const char *
 lnc_strerror(int err)
 {
-
 #ifdef vms 
 	if(err == EVMSERR)
 	{
@@ -229,8 +230,6 @@ lnc_strerror(int err)
             return build_opendap_error_message("Not authorized");
         case OP_EREADFAIL:
             return build_opendap_error_message("Could not read file");
-        case OP_EREAD_FAIL:
-            return build_opendap_error_message("Could not read file");        
 	}
 	/* default */
 	return unknown;
