@@ -2,13 +2,16 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: llibvers.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: llibvers.c 14988 2006-11-01 22:21:09Z jimg $ */
 
+#include <config.h>
 #include "lnetcdf.h"
 
 /*
- * A version string.
+ * A version string. This whole function is not needed in netCDF-4,
+ * which has it's own version of this function.
  */
+#ifndef USE_NETCDF4
 #define SKIP_LEADING_GARBAGE 33	/* # of chars prior to the actual version */
 #define XSTRING(x)	#x
 #define STRING(x)	XSTRING(x)
@@ -21,6 +24,8 @@ lnc_inq_libvers(void)
 	return &nc_libvers[SKIP_LEADING_GARBAGE];
 }
 
+#endif /* USE_NETCDF4*/
+
 #if MAKE_PROGRAM /* TEST JIG */
 #include <stdio.h>
 
