@@ -30,7 +30,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: lnetcdf.h 14988 2006-11-01 22:21:09Z jimg $" */
+/* "$Id: lnetcdf.h 17638 2008-01-07 20:47:29Z jimg $" */
 
 #ifndef _NETCDF_
 #define _NETCDF_
@@ -168,7 +168,7 @@ typedef enum {
  * values from the system errno.h.
  */
 
-#define NC_ISSYSERR(err)	((err) > 0)
+#define NC_ISSYSERR(err)	((err) > 0 && (err) < 1000)
 
 #define	NC_NOERR	0	/* No Error */
 
@@ -218,8 +218,7 @@ typedef enum {
 #define OP_ENOVAR 1004 /** (no_such_variable) */
 #define OP_EMALFORMED 1005 /** (malformed_expr) */
 #define OP_EAUTH 1006 /** (no_authorization) */
-#define OP_EREADFAIL 1007 /** (can_not_read_file) */
-#define OP_EREAD_FAIL 1008 /** (cannot_read_file) */
+#define OP_EREADFAIL 1007 /** (cannot_read_file) */
 
 /*
  * The Interface
