@@ -2,10 +2,11 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: lv1hpg.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: lv1hpg.c 14988 2006-11-01 22:21:09Z jimg $ */
 
 #include "lnc.h"
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include "lrnd.h"
@@ -13,9 +14,12 @@
 
 /*
  * This module defines the external representation
- * of the "header" of a netcdf version one file.
+ * of the "header" of a netcdf version one file and
+ * the version two variant that uses 64-bit file 
+ * offsets instead of the 32-bit file offsets in version 
+ * one files.
  * For each of the components of the NC structure,
- * There are (static) ncx_len_XXX(), ncx_put_XXX()
+ * There are (static) ncx_len_XXX(), v1h_put_XXX()
  * and v1h_get_XXX() functions. These define the
  * external representation of the components.
  * The exported entry points for the whole NC structure
@@ -27,7 +31,8 @@
  * "magic number" at beginning of file: 0x43444601 (big endian)
  * assert(sizeof(ncmagic) % X_ALIGN == 0);
  */
-static const schar ncmagic[] = {'C', 'D', 'F', 0x01};
+static const schar ncmagic[] = {'C', 'D', 'F', 0x02};
+static const schar ncmagic1[] = {'C', 'D', 'F', 0x01};
 
 
 /*
@@ -44,6 +49,7 @@ typedef struct v1hs {
 	off_t offset;	/* argument to nciop->get() */
 	size_t extent;	/* argument to nciop->get() */
 	int flags;	/* set to RGN_WRITE for write */
+        int version;    /* format variant: NC_FORMAT_CLASSIC or NC_FORMAT_64BIT */
 	void *base;	/* beginning of current buffer */
 	void *pos;	/* current position in buffer */
 	void *end;	/* end of current buffer = base + extent */
@@ -122,6 +128,7 @@ fprintf(stderr, "nextread %lu, remaining %lu\n",
 
 /* End v1hs */
 
+/* Write a size_t to the header */
 static int
 v1h_put_size_t(v1hs *psp, const size_t *sp)
 {
@@ -132,6 +139,7 @@ v1h_put_size_t(v1hs *psp, const size_t *sp)
 }
 
 
+/* Read a size_t from the header */
 static int
 v1h_get_size_t(v1hs *gsp, size_t *sp)
 {
@@ -146,6 +154,7 @@ v1h_get_size_t(v1hs *gsp, size_t *sp)
 
 #define X_SIZEOF_NC_TYPE X_SIZEOF_INT
 
+/* Write a nc_type to the header */
 static int
 v1h_put_nc_type(v1hs *psp, const nc_type *typep)
 {
@@ -159,6 +168,7 @@ v1h_put_nc_type(v1hs *psp, const nc_type *typep)
 }
 
 
+/* Read a nc_type from the header */
 static int
 v1h_get_nc_type(v1hs *gsp, nc_type *typep)
 {
@@ -189,6 +199,7 @@ v1h_get_nc_type(v1hs *gsp, nc_type *typep)
 
 #define X_SIZEOF_NCTYPE X_SIZEOF_INT
 
+/* Write a NCtype to the header */
 static int
 v1h_put_NCtype(v1hs *psp, NCtype type)
 {
@@ -201,6 +212,7 @@ v1h_put_NCtype(v1hs *psp, NCtype type)
 	return status;
 }
 
+/* Read a NCtype from the header */
 static int
 v1h_get_NCtype(v1hs *gsp, NCtype *typep)
 {
@@ -245,6 +257,7 @@ ncx_len_NC_string(const NC_string *ncstrp)
 }
 
 
+/* Write a NC_string to the header */
 static int
 v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 {
@@ -268,6 +281,7 @@ v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 }
 
 
+/* Read a NC_string from the header */
 static int
 v1h_get_NC_string(v1hs *gsp, NC_string **ncstrpp)
 {
@@ -334,6 +348,7 @@ ncx_len_NC_dim(const NC_dim *dimp)
 }
 
 
+/* Write a NC_dim to the header */
 static int
 v1h_put_NC_dim(v1hs *psp, const NC_dim *dimp)
 {
@@ -350,6 +365,7 @@ v1h_put_NC_dim(v1hs *psp, const NC_dim *dimp)
 	return ENOERR;
 }
 
+/* Read a NC_dim from the header */
 static int
 v1h_get_NC_dim(v1hs *gsp, NC_dim **dimpp)
 {
@@ -385,6 +401,7 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_dimarray? */
 static size_t
 ncx_len_NC_dimarray(const NC_dimarray *ncap)
 {
@@ -405,6 +422,7 @@ ncx_len_NC_dimarray(const NC_dimarray *ncap)
 }
 
 
+/* Write a NC_dimarray to the header */
 static int
 v1h_put_NC_dimarray(v1hs *psp, const NC_dimarray *ncap)
 {
@@ -459,6 +477,7 @@ v1h_put_NC_dimarray(v1hs *psp, const NC_dimarray *ncap)
 }
 
 
+/* Read a NC_dimarray from the header */
 static int
 v1h_get_NC_dimarray(v1hs *gsp, NC_dimarray *ncap)
 {
@@ -569,6 +588,7 @@ v1h_put_NC_attrV(v1hs *psp, const NC_attr *attrp)
 	return ENOERR;
 }
 
+/* Write a NC_attr to the header */
 static int
 v1h_put_NC_attr(v1hs *psp, const NC_attr *attrp)
 {
@@ -608,8 +628,6 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 	void *value = attrp->xvalue;
 	size_t nget; 
 
-	assert(gsp->extent % X_ALIGN == 0);
-	
 	do {
 		nget = MIN(perchunk, remaining);
 	
@@ -629,6 +647,7 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 }
 
 
+/* Read a NC_attr from the header */
 static int
 v1h_get_NC_attr(v1hs *gsp, NC_attr **attrpp)
 {
@@ -674,6 +693,7 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_attrarray? */
 static size_t
 ncx_len_NC_attrarray(const NC_attrarray *ncap)
 {
@@ -694,6 +714,7 @@ ncx_len_NC_attrarray(const NC_attrarray *ncap)
 }
 
 
+/* Write a NC_attrarray to the header */
 static int
 v1h_put_NC_attrarray(v1hs *psp, const NC_attrarray *ncap)
 {
@@ -748,6 +769,7 @@ v1h_put_NC_attrarray(v1hs *psp, const NC_attrarray *ncap)
 }
 
 
+/* Read a NC_attrarray from the header */
 static int
 v1h_get_NC_attrarray(v1hs *gsp, NC_attrarray *ncap)
 {
@@ -803,11 +825,12 @@ v1h_get_NC_attrarray(v1hs *gsp, NC_attrarray *ncap)
 NC_xlen_var(vpp)
  */
 static size_t
-ncx_len_NC_var(const NC_var *varp)
+ncx_len_NC_var(const NC_var *varp, size_t sizeof_off_t)
 {
 	size_t sz;
 
 	assert(varp != NULL);
+	assert(sizeof_off_t != 0);
 
 	sz = ncx_len_NC_string(varp->name);
 	sz += X_SIZEOF_SIZE_T; /* ndims */
@@ -815,12 +838,13 @@ ncx_len_NC_var(const NC_var *varp)
 	sz += ncx_len_NC_attrarray(&varp->attrs);
 	sz += X_SIZEOF_NC_TYPE; /* type */
 	sz += X_SIZEOF_SIZE_T; /* len */
-	sz += X_SIZEOF_OFF_T; /* begin */
+	sz += sizeof_off_t; /* begin */
 
 	return(sz);
 }
 
 
+/* Write a NC_var to the header */
 static int
 v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 {
@@ -854,10 +878,10 @@ v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 	if(status != ENOERR)
 		return status;
 
-	status = check_v1hs(psp, X_SIZEOF_OFF_T);
+	status = check_v1hs(psp, psp->version == 1 ? 4 : 8);
 	if(status != ENOERR)
 		 return status;
-	status = ncx_put_off_t(&psp->pos, &varp->begin);
+	status = ncx_put_off_t(&psp->pos, &varp->begin, psp->version == 1 ? 4 : 8);
 	if(status != ENOERR)
 		return status;
 
@@ -865,6 +889,7 @@ v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 }
 
 
+/* Read a NC_var from the header */
 static int
 v1h_get_NC_var(v1hs *gsp, NC_var **varpp)
 {
@@ -908,11 +933,11 @@ v1h_get_NC_var(v1hs *gsp, NC_var **varpp)
 	if(status != ENOERR)
 		 goto unwind_alloc;
 
-	status = check_v1hs(gsp, X_SIZEOF_OFF_T);
+	status = check_v1hs(gsp, gsp->version == 1 ? 4 : 8);
 	if(status != ENOERR)
 		 goto unwind_alloc;
 	status = ncx_get_off_t((const void **)&gsp->pos,
-			&varp->begin);
+			       &varp->begin, gsp->version == 1 ? 4 : 8);
 	if(status != ENOERR)
 		 goto unwind_alloc;
 	
@@ -929,8 +954,9 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_vararray? */
 static size_t
-ncx_len_NC_vararray(const NC_vararray *ncap)
+ncx_len_NC_vararray(const NC_vararray *ncap, size_t sizeof_off_t)
 {
 	size_t xlen = X_SIZEOF_NCTYPE;	/* type */
 	xlen += X_SIZEOF_SIZE_T;	/* count */
@@ -942,13 +968,14 @@ ncx_len_NC_vararray(const NC_vararray *ncap)
 		const NC_var *const *const end = &vpp[ncap->nelems];
 		for( /*NADA*/; vpp < end; vpp++)
 		{
-			xlen += ncx_len_NC_var(*vpp);
+			xlen += ncx_len_NC_var(*vpp, sizeof_off_t);
 		}
 	}
 	return xlen;
 }
 
 
+/* Write a NC_vararray to the header */
 static int
 v1h_put_NC_vararray(v1hs *psp, const NC_vararray *ncap)
 {
@@ -1003,6 +1030,7 @@ v1h_put_NC_vararray(v1hs *psp, const NC_vararray *ncap)
 }
 
 
+/* Read a NC_vararray from the header */
 static int
 v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
 {
@@ -1059,8 +1087,8 @@ v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
  * Recompute the shapes of all variables
  * Sets ncp->begin_var to start of first variable.
  * Sets ncp->begin_rec to start of first record variable.
- * Returns -1 on error. The only possible error is an reference
- * to a non existent dimension, which would occur for a corrupt
+ * Returns -1 on error. The only possible error is a reference
+ * to a non existent dimension, which could occur for a corrupted
  * netcdf file.
  */
 static int
@@ -1091,12 +1119,13 @@ NC_computeshapes(NC *ncp)
 				first_rec = *vpp;
 			ncp->recsize += (*vpp)->len;
 		}
-		else if(first_var == NULL)
+		else
 		{
-			first_var = *vpp;
+		        if(first_var == NULL)
+			        first_var = *vpp;
 			/*
 			 * Overwritten each time thru.
-			 * Usually overwritten in first_rec != NULL clause.
+			 * Usually overwritten in first_rec != NULL clause below.
 			 */
 			ncp->begin_rec = (*vpp)->begin + (off_t)(*vpp)->len;
 		}
@@ -1131,8 +1160,31 @@ NC_computeshapes(NC *ncp)
 }
 
 
+/*
+ * Return actual unpadded length (in bytes) of a variable, which
+ * doesn't include any extra padding used for alignment.  For a record
+ * variable, this is the length in bytes of one record's worth of that
+ * variable's data.
+ */
+static off_t
+NC_var_unpadded_len(const NC_var *varp, const NC_dimarray *dims)
+{
+    size_t *shp;
+    off_t product = 1;
+    
+    if(varp->ndims != 0) {
+	for(shp = varp->shape + varp->ndims -1; shp >= varp->shape; shp--) {
+	    if(!(shp == varp->shape && IS_RECVAR(varp)))
+		product *= *shp;
+	}
+    }
+    product = product * varp->xsz;
+    return product;
+}
+
+/* How much space in the header is required for the NC data structure? */
 size_t
-ncx_len_NC(const NC *ncp)
+ncx_len_NC(const NC *ncp, size_t sizeof_off_t)
 {
 	size_t xlen = sizeof(ncmagic);
 
@@ -1141,12 +1193,13 @@ ncx_len_NC(const NC *ncp)
 	xlen += X_SIZEOF_SIZE_T; /* numrecs */
 	xlen += ncx_len_NC_dimarray(&ncp->dims);
 	xlen += ncx_len_NC_attrarray(&ncp->attrs);
-	xlen += ncx_len_NC_vararray(&ncp->vars);
+	xlen += ncx_len_NC_vararray(&ncp->vars, sizeof_off_t);
 
 	return xlen;
 }
 
 
+/* Write the file header */
 int
 ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 {
@@ -1160,6 +1213,11 @@ ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 	ps.nciop = ncp->nciop;
 	ps.flags = RGN_WRITE;
 
+	if (ncp->flags & NC_64BIT_OFFSET)
+	  ps.version = 2;
+	else 
+	  ps.version = 1;
+
 	if(xpp == NULL)
 	{
 		/*
@@ -1198,7 +1256,10 @@ ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 		ps.end = (char *)ps.base + ps.extent;
 	}
 
-	status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic), ncmagic);
+	if (ps.version == 2)
+	  status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic), ncmagic);
+	else
+	  status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic1), ncmagic1);
 	if(status != ENOERR)
 		goto release;
 
@@ -1230,6 +1291,7 @@ release:
 }
 
 
+/* Make the in-memory NC structure from reading the file header */
 int
 lnc_get_NC(NC *ncp)
 {
@@ -1244,6 +1306,7 @@ lnc_get_NC(NC *ncp)
 	gs.offset = 0; /* beginning of file */
 	gs.extent = 0;
 	gs.flags = 0;
+	gs.version = 0;
 	gs.base = NULL;
 	gs.pos = gs.base;
 
@@ -1251,15 +1314,23 @@ lnc_get_NC(NC *ncp)
 		/*
 		 * Come up with a reasonable stream read size.
 		 */
-		size_t extent = ncp->xsz;
+	        off_t filesize;
+		size_t extent = MIN_NC_XSZ;
+		
+		extent = ncp->xsz;
 		if(extent <= MIN_NC_XSZ)
 		{
+		        status = ncio_filesize(ncp->nciop, &filesize);
+			if(status)
+			    return status;
 			/* first time read */
 			extent = ncp->chunk;
 			/* Protection for when ncp->chunk is huge;
 			 * no need to read hugely. */
 	      		if(extent > 4096)
 				extent = 4096;
+			if(extent > filesize)
+			        extent = filesize;
 		}
 		else if(extent > ncp->chunk)
 		{
@@ -1291,11 +1362,27 @@ lnc_get_NC(NC *ncp)
 		if(status != ENOERR)
 			goto unwind_get;
 	
-		if(memcmp(magic, ncmagic, sizeof(ncmagic)) != 0)
+		if(memcmp(magic, ncmagic, sizeof(ncmagic)-1) != 0)
 		{
 			status = NC_ENOTNC;
 			goto unwind_get;
 		}
+		/* Check version number in last byte of magic */
+		if (magic[sizeof(ncmagic)-1] == 0x1) {
+		  gs.version = 1;
+		} else if (magic[sizeof(ncmagic)-1] == 0x2) {
+		  gs.version = 2;
+		  fSet(ncp->flags, NC_64BIT_OFFSET);
+		  /* Now we support version 2 file access on non-LFS systems -- rkr */
+#if 0
+		  if (sizeof(off_t) != 8) {
+		    fprintf(stderr, "NETCDF WARNING: Version 2 file on 32-bit system.\n");
+		  }
+#endif
+		} else {
+			status = NC_ENOTNC;
+			goto unwind_get;
+		}
 	}
 	
 	{
@@ -1320,9 +1407,11 @@ lnc_get_NC(NC *ncp)
 	if(status != ENOERR)
 		goto unwind_get;
 		
-	ncp->xsz = ncx_len_NC(ncp);
+	ncp->xsz = ncx_len_NC(ncp, (gs.version == 1) ? 4 : 8);
 
 	status = NC_computeshapes(ncp);
+	if(status != ENOERR)
+		goto unwind_get;
 
 unwind_get:
 	(void) rel_v1hs(&gs);
