@@ -2,9 +2,15 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: lv2i.c 11906 2005-08-08 19:51:43Z root $ */
+/* $Id: lv2i.c 15125 2006-11-21 23:12:00Z jimg $ */
 
-#include "lnc.h"		/* nc.h --> lnc,h 03/05/04 jhrg */
+#include <config.h>
+#include <stdlib.h>
+#include <netcdf.h>
+
+#if 0
+#include "lnc.h"		/* nc.h --> lnc.h 03/05/04 jhrg */
+#endif
 
 #ifndef NO_NETCDF_2
 
@@ -67,12 +73,279 @@ nvdims(int ncid, int varid)
 
 #endif
 
+typedef signed char schar;
+
+/*
+ * Computes number of record variables in an open netCDF file, and an array of
+ * the record variable ids, if the array parameter is non-null.
+ */
+static int
+numrecvars(int ncid, int *nrecvarsp, int *recvarids)
+{
+    int status;
+    int nvars = 0;
+    int ndims = 0;
+    int nrecvars = 0;
+    int varid;
+    int recdimid;
+    int dimids[MAX_NC_DIMS];
+
+    status = nc_inq_nvars(ncid, &nvars); 
+    if(status != NC_NOERR)
+	return status;
+
+    status = nc_inq_unlimdim(ncid, &recdimid); 
+    if(status != NC_NOERR)
+	return status;
+
+    if (recdimid == -1) {
+	*nrecvarsp = 0;
+	return NC_NOERR;
+    }
+    nrecvars = 0;
+    for (varid = 0; varid < nvars; varid++) {
+	status = nc_inq_varndims(ncid, varid, &ndims); 
+	if(status != NC_NOERR)
+	    return status;
+	status = nc_inq_vardimid(ncid, varid, dimids); 
+	if(status != NC_NOERR)
+	    return status;
+	if (ndims > 0 && dimids[0] == recdimid) {
+	    if (recvarids != NULL)
+	      recvarids[nrecvars] = varid;
+	    nrecvars++;
+	}
+    }
+    *nrecvarsp = nrecvars;
+    return NC_NOERR;
+}
+
+
+/*
+ * Computes record size (in bytes) of the record variable with a specified
+ * variable id.  Returns size as 0 if not a record variable.
+ */
+static int
+ncrecsize(int ncid, int varid, size_t *recsizep)
+{
+    int status;
+    int recdimid;
+    nc_type type;
+    int ndims;
+    int dimids[MAX_NC_DIMS];
+    int id;
+    size_t size;
+
+    *recsizep = 0;
+    status = nc_inq_unlimdim(ncid, &recdimid); 
+    if(status != NC_NOERR)
+	return status;
+    status = nc_inq_vartype(ncid, varid, &type); 
+    if(status != NC_NOERR)
+	return status;
+    status = nc_inq_varndims(ncid, varid, &ndims); 
+    if(status != NC_NOERR)
+	return status;
+    status = nc_inq_vardimid(ncid, varid, dimids); 
+    if(status != NC_NOERR)
+	return status;
+    if (ndims == 0 || dimids[0] != recdimid) {
+	return NC_NOERR;
+    }
+    size = nctypelen(type);
+    for (id = 1; id < ndims; id++) {
+	size_t len;
+	status = nc_inq_dimlen(ncid, dimids[id], &len);
+	if(status != NC_NOERR)
+		return status;
+	size *= len;
+    }
+    *recsizep = size;
+    return NC_NOERR;
+}
+
+
+/*
+ * Retrieves the dimension sizes of a variable with a specified variable id in
+ * an open netCDF file.  Returns -1 on error.
+ */
+static int
+dimsizes(int ncid, int varid, size_t *sizes)
+{
+    int status;
+    int ndims;
+    int id;
+    int dimids[MAX_NC_DIMS];
+
+    status = nc_inq_varndims(ncid, varid, &ndims); 
+    if(status != NC_NOERR)
+	return status;
+    status = nc_inq_vardimid(ncid, varid, dimids); 
+    if(status != NC_NOERR)
+	return status;
+    if (ndims == 0 || sizes == NULL)
+      return NC_NOERR;
+    for (id = 0; id < ndims; id++) {
+	size_t len;
+	status = nc_inq_dimlen(ncid, dimids[id], &len);
+	if(status != NC_NOERR)
+		return status;
+	sizes[id] = len;
+    }
+    return NC_NOERR;
+}
+
+
+/*
+ * Retrieves the number of record variables, the record variable ids, and the
+ * record size of each record variable.  If any pointer to info to be returned
+ * is null, the associated information is not returned.  Returns -1 on error.
+ */
+int
+lnc_inq_rec(
+	int ncid,
+	size_t *nrecvarsp,
+	int *recvarids,
+	size_t *recsizes)
+{
+    int status;
+    int nvars = 0;
+    int recdimid;
+    int varid;
+    int rvarids[MAX_NC_VARS];
+    int nrvars = 0;
+
+    status = nc_inq_nvars(ncid, &nvars); 
+    if(status != NC_NOERR)
+	return status;
+
+    status = nc_inq_unlimdim(ncid, &recdimid); 
+    if(status != NC_NOERR)
+	return status;
+
+    *nrecvarsp = 0;
+    if (recdimid == -1)
+	return NC_NOERR;
+    
+    status = numrecvars(ncid, &nrvars, rvarids);
+    if(status != NC_NOERR)
+	return status;
+
+    if (nrecvarsp != NULL)
+	*nrecvarsp = nrvars;
+    if (recvarids != NULL)
+	for (varid = 0; varid < nrvars; varid++)
+	    recvarids[varid] = rvarids[varid];
+
+    if (recsizes != NULL)
+	for (varid = 0; varid < nrvars; varid++) {
+	    size_t rsize;
+	    status = ncrecsize(ncid, rvarids[varid], &rsize);
+	    if (status != NC_NOERR)
+		return status;
+	    recsizes[varid] = rsize;
+	}
+	return NC_NOERR;
+}
+
+
+/*
+ * Write one record's worth of data, except don't write to variables for which
+ * the address of the data to be written is NULL.  Return -1 on error.  This is
+ * the same as the ncrecput() in the library, except that can handle errors
+ * better.
+ */
+int
+lnc_put_rec(
+	int ncid,
+	size_t recnum,
+	void* const* datap)
+{
+    int status;
+    int varid;
+    int rvarids[MAX_NC_VARS];
+    int nrvars;
+    size_t start[MAX_NC_DIMS];
+    size_t edges[MAX_NC_DIMS];
+
+    status = numrecvars(ncid, &nrvars, rvarids);
+    if(status != NC_NOERR)
+	return status;
+
+    if (nrvars == 0)
+      return NC_NOERR;
+
+    start[0] = recnum;
+    for (varid = 1; varid < nrvars; varid++)
+	start[varid] = 0;
+
+    for (varid = 0; varid < nrvars; varid++) {
+	if (datap[varid] != NULL) {
+	    status = dimsizes(ncid, rvarids[varid], edges);
+	    if(status != NC_NOERR)
+		return status;
+
+	    edges[0] = 1;		/* only 1 record's worth */
+	    status = nc_put_vara(ncid, rvarids[varid], start, edges, datap[varid]);
+	    if(status != NC_NOERR)
+		return status;
+	}
+    }    
+    return 0;
+}
+
+
+/*
+ * Read one record's worth of data, except don't read from variables for which
+ * the address of the data to be read is null.  Return -1 on error.  This is
+ * the same as the ncrecget() in the library, except that can handle errors
+ * better.
+ */
+int
+lnc_get_rec(
+	int ncid,
+	size_t recnum,
+	void **datap)
+{
+    int status;
+    int varid;
+    int rvarids[MAX_NC_VARS];
+    int nrvars;
+    size_t start[MAX_NC_DIMS];
+    size_t edges[MAX_NC_DIMS];
+
+    status = numrecvars(ncid, &nrvars, rvarids);
+    if(status != NC_NOERR)
+	return status;
+
+    if (nrvars == 0)
+      return NC_NOERR;
+
+    start[0] = recnum;
+    for (varid = 1; varid < nrvars; varid++)
+	start[varid] = 0;
+
+    for (varid = 0; varid < nrvars; varid++) {
+	if (datap[varid] != NULL) {
+	    status = dimsizes(ncid, rvarids[varid], edges);
+	    if(status != NC_NOERR)
+		return status;
+	    edges[0] = 1;		/* only 1 record's worth */
+	    status = nc_get_vara(ncid, rvarids[varid], start, edges, datap[varid]);
+	    if(status != NC_NOERR)
+		return status;
+	}
+    }    
+    return 0;
+}
+
 
 /* Begin globals */
 
 /*
  * Error code
  */
+#if !DLL_NETCDF /* define when library is not a DLL */
 int ncerr = NC_NOERR ;
 
 
@@ -81,7 +354,7 @@ int ncerr = NC_NOERR ;
  * They call exit() when NC_FATAL bit is on.
  */
 int ncopts = (NC_FATAL | NC_VERBOSE) ;
-
+#endif
 
 /*
  * Backward compatibility for the version 2 fortran jackets
@@ -283,7 +556,13 @@ ncdimdef(
 )
 {
 	int dimid;
-	const int status =  nc_def_dim(ncid, name, (size_t)length, &dimid);
+	int status;
+	if(length < 0) {
+	    status = NC_EDIMSIZE;
+	    nc_advise("ncdimdef", status, "ncid %d", ncid);
+	    return -1;
+	}
+	status =  nc_def_dim(ncid, name, (size_t)length, &dimid);
 	if(status != NC_NOERR)
 	{
 		nc_advise("ncdimdef", status, "ncid %d", ncid);
@@ -835,37 +1114,6 @@ ncattdel(
 
 #endif /* NO_NETCDF_2 */
 
-/*
- *  This is how much space is required by the user, as in
- *
- *   vals = malloc(nel * nctypelen(var.type));
- *   ncvarget(cdfid, varid, cor, edg, vals);
- */
-int
-nctypelen(nc_type type) 
-{
-	switch(type){
-	case NC_BYTE :
-	case NC_CHAR :
-		return((int)sizeof(char));
-	case NC_SHORT :
-		return(int)(sizeof(short));
-	case NC_INT :
-		return((int)sizeof(int));
-	case NC_FLOAT :
-		return((int)sizeof(float));
-	case NC_DOUBLE : 
-		return((int)sizeof(double));
-	}
-
-#ifndef NO_NETCDF_2
-	/* else */
-	nc_advise("nctypelen", NC_EBADTYPE, "Unknown type %d",
-		(int)type);
-#endif /* NO_NETCDF_2 */
-	return -1;
-}
-
 #ifndef NO_NETCDF_2
 
 int
