@@ -1,5 +1,5 @@
 /*
- * Copyright 1993-1996 University Corporation for Atmospheric Research/Unidata
+ * Copyright 1993-2005 University Corporation for Atmospheric Research/Unidata
  * 
  * Portions of this software were developed by the Unidata Program at the 
  * University Corporation for Atmospheric Research.
@@ -30,8 +30,8 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/* "$Id: netcdf.h 11906 2005-08-08 19:51:43Z root $" */
-
+/* "$Id: netcdf.h 14988 2006-11-01 22:21:09Z jimg $" */
+
 #ifndef _NETCDF_
 #define _NETCDF_
 
@@ -78,21 +78,43 @@ typedef enum {
  * will be used as the fill value for that variable.
  */
 #define _FillValue	"_FillValue"
+#define NC_FILL		0	/* argument to ncsetfill to clear NC_NOFILL */
+#define NC_NOFILL	0x100	/* Don't fill data section an records */
 
+/*
+ * 'mode' flags for ncopen
+ */
+#define NC_NOWRITE	0	/* default is read only */
+#define NC_WRITE    	0x1	/* read & write */
+
+/*
+ * 'mode' flags for nccreate
+ */
+#define NC_CLOBBER	0
+#define NC_NOCLOBBER	0x4	/* Don't destroy existing file on create */
+#define NC_64BIT_OFFSET 0x0200  /* Use large (64-bit) file offsets */
 
 /*
  * 'mode' flags for nccreate and ncopen
  */
-#define NC_NOWRITE	0	/* default is read only */
-#define NC_WRITE    	0x1	/* read & write */
-#define NC_CLOBBER	0
-#define NC_NOCLOBBER	0x4	/* Don't destroy existing file on create */
-#define NC_FILL		0	/* argument to ncsetfill to clear NC_NOFILL */
-#define NC_NOFILL	0x100	/* Don't fill data section an records */
-#define NC_LOCK		0x0400	/* Use locking if available */
 #define NC_SHARE	0x0800	/* Share updates, limit cacheing */
+#define NC_STRICT_NC3  (0x8)
+
+/* The following flag currently is ignored, but use in
+ * nc_open() or nc_create() may someday support use of advisory
+ * locking to prevent multiple writers from clobbering a file 
+ */
+#define NC_LOCK		0x0400	/* Use locking if available */
+
+/*
+ * Starting with version 3.6, there are different format netCDF
+ * files. 4.0 instroduces the third one.
+ */
+#define NC_FORMAT_CLASSIC (1)
+#define NC_FORMAT_64BIT   (2)
+#define NC_FORMAT_NETCDF4 (3)
+#define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_STRICT_NC3. */
 
-
 /*
  * Let nc__create() or nc__open() figure out
  * as suitable chunk size.
@@ -114,40 +136,43 @@ typedef enum {
  */
 #define NC_GLOBAL -1
 
+/* These are in support of the coordinate axis stuff. */
+#define NC_NOAXISTYPE 0
+#define NC_LATITUDE 1
+#define NC_LONGITUDE 2
+#define NC_GEOX 3
+#define NC_GEOY 4
+#define NC_GEOZ 5
+#define NC_HEIGHT_UP 6
+#define NC_HEIGHT_DOWN 7
+#define NC_PRESSURE 8
+#define NC_TIME 9
+#define NC_RADAZ 10
+#define NC_RADEL 11
+#define NC_RADDIST 12
 
 /*
  * These maximums are enforced by the interface, to facilitate writing
  * applications and utilities.  However, nothing is statically allocated to
  * these sizes internally.
  */
-#if 0
-#define NC_MAX_DIMS       100      /* max dimensions per file */
-#define NC_MAX_ATTRS     2000     /* max global or per variable attributes */
-#define NC_MAX_VARS        2000     /* max variables per file */
-#define NC_MAX_NAME       128      /* max length of a name */
-#define NC_MAX_VAR_DIMS     NC_MAX_DIMS /* max per variable dimensions */
-#else
-#define NC_MAX_DIMS     1024     /* max dimensions per file */
-#define NC_MAX_ATTRS    4096     /* max global or per variable attributes */
-#define NC_MAX_VARS     4096     /* max variables per file */
-#define NC_MAX_NAME     1024      /* max length of a name */
-#define NC_MAX_VAR_DIMS NC_MAX_DIMS /* max per variable dimensions */
-#endif
-
+#define NC_MAX_DIMS	1024	 /* max dimensions per file */
+#define NC_MAX_ATTRS	8192	 /* max global or per variable attributes */
+#define NC_MAX_VARS	8192	 /* max variables per file */
+#define NC_MAX_NAME	256	 /* max length of a name */
+#define NC_MAX_VAR_DIMS	NC_MAX_DIMS /* max per variable dimensions */
+
 /*
  * The netcdf version 3 functions all return integer error status.
  * These are the possible values, in addition to certain
  * values from the system errno.h.
- * 
- * I've added error codes above 1000 as opendap error codes. These
- * should all fall below the codes in errno.h; I'm using 1000++ becuase
- * those are the values used by the DAP library. jhrg 3/2/05
  */
 
-#define NC_ISSYSERR(err)        (((err) > 0) && ((err) < 1000))
+#define NC_ISSYSERR(err)	((err) > 0)
 
 #define	NC_NOERR	0	/* No Error */
 
+#define NC2_ERR         (-1)    /* Returned for all errors in the v2 API. */
 #define	NC_EBADID	(-33)	/* Not a netcdf id */
 #define	NC_ENFILE	(-34)	/* Too many netcdfs open */
 #define	NC_EEXIST	(-35)	/* netcdf file exists && NC_NOCLOBBER */
@@ -172,7 +197,7 @@ typedef enum {
 #define NC_EUNLIMIT    	(-54)	/* NC_UNLIMITED size already in use */
 #define NC_ENORECVARS  	(-55)	/* nc_rec op when there are no record vars */
 #define NC_ECHAR	(-56)	/* Attempt to convert between text & numbers */
-#define NC_EEDGE	(-57)	/* Edge+start exceeds dimension bound */
+#define NC_EEDGE	(-57)	/* Start+count exceeds dimension bound */
 #define NC_ESTRIDE	(-58)	/* Illegal stride */
 #define NC_EBADNAME	(-59)	/* Attribute or variable name
                                          contains illegal characters */
@@ -180,17 +205,11 @@ typedef enum {
 #define NC_ERANGE	(-60)	/* Math result not representable */
 #define NC_ENOMEM	(-61)	/* Memory allocation (malloc) failure */
 
-/* These are specific to the OPeNDAP netCDF 3.x client library. */
-#define OP_EUNDEF 1000 /** Undefined error code (undefined_error) */
-#define OP_EUNKNOWN 1001 /** Unknown error (unknown_error) */
-#define OP_INTERNAL 1002 /** Internal server error (internal_error) */
-#define OP_ENOFILE 1003 /** (no_such_file) */
-#define OP_ENOVAR 1004 /** (no_such_variable) */
-#define OP_EMALFORMED 1005 /** (malformed_expr) */
-#define OP_EAUTH 1006 /** (no_authorization) */
-#define OP_EREADFAIL 1007 /** (can_not_read_file) */
-#define OP_EREAD_FAIL 1008 /** (cannot_read_file) */
-
+#define NC_EVARSIZE     (-62)   /* One or more variable sizes violate
+				   format constraints */ 
+#define NC_EDIMSIZE     (-63)   /* Invalid dimension size */
+#define NC_ETRUNC       (-64)   /* File likely truncated or possibly corrupted */
+
 /*
  * The Interface
  */
@@ -203,12 +222,19 @@ typedef enum {
 #  else
 #   define MSC_EXTRA __declspec(dllimport)
 #  endif
+#include <io.h>
+#define lseek _lseeki64
+#define off_t __int64
 #else
 #define MSC_EXTRA
 #endif	/* defined(DLL_NETCDF) */
 
 # define EXTERNL extern MSC_EXTRA
 
+#if defined(DLL_NETCDF) /* define when library is a DLL */
+MSC_EXTRA int ncerr;
+MSC_EXTRA int ncopts;
+#endif
 EXTERNL const char *
 nc_inq_libvers(void);
 
@@ -251,9 +277,6 @@ nc_abort(int ncid);
 EXTERNL int
 nc_close(int ncid);
 
-EXTERNL int
-nc_delete(const char * path);
-
 EXTERNL int
 nc_inq(int ncid, int *ndimsp, int *nvarsp, int *nattsp, int *unlimdimidp);
 
@@ -268,7 +291,13 @@ nc_inq_natts(int ncid, int *nattsp);
 
 EXTERNL int 
 nc_inq_unlimdim(int ncid, int *unlimdimidp);
-
+
+EXTERNL int
+nc_set_default_format(int format, int *old_formatp);
+
+EXTERNL int
+nc_inq_format(int ncid, int *formatp);
+
 /* Begin _dim */
 
 EXTERNL int
@@ -294,7 +323,7 @@ nc_rename_dim(int ncid, int dimid, const char *name);
 
 EXTERNL int
 nc_inq_att(int ncid, int varid, const char *name,
-	 nc_type *xtypep, size_t *lenp);
+	   nc_type *xtypep, size_t *lenp);
 
 EXTERNL int 
 nc_inq_attid(int ncid, int varid, const char *name, int *idp);
@@ -317,12 +346,19 @@ nc_rename_att(int ncid, int varid, const char *name, const char *newname);
 EXTERNL int
 nc_del_att(int ncid, int varid, const char *name);
 
-/* End _att */
+/* End _att */
 /* Begin {put,get}_att */
 
+EXTERNL int
+nc_put_att(int ncid, int varid, const char *name, nc_type datatype,
+	   size_t len, const void *value);
+
+EXTERNL int
+nc_get_att(int ncid, int varid, const char *name, void *value);
+
 EXTERNL int
 nc_put_att_text(int ncid, int varid, const char *name,
-	size_t len, const char *op);
+		size_t len, const char *op);
 
 EXTERNL int
 nc_get_att_text(int ncid, int varid, const char *name, char *ip);
@@ -376,16 +412,16 @@ nc_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 EXTERNL int
 nc_get_att_double(int ncid, int varid, const char *name, double *ip);
 
-/* End {put,get}_att */
+/* End {put,get}_att */
 /* Begin _var */
 
 EXTERNL int
-nc_def_var(int ncid, const char *name,
-	 nc_type xtype, int ndims, const int *dimidsp, int *varidp);
+nc_def_var(int ncid, const char *name, nc_type xtype, int ndims, 
+	   const int *dimidsp, int *varidp);
 
 EXTERNL int
-nc_inq_var(int ncid, int varid, char *name,
-	 nc_type *xtypep, int *ndimsp, int *dimidsp, int *nattsp);
+nc_inq_var(int ncid, int varid, char *name, nc_type *xtypep, 
+	   int *ndimsp, int *dimidsp, int *nattsp);
 
 EXTERNL int
 nc_inq_varid(int ncid, const char *name, int *varidp);
@@ -415,9 +451,15 @@ nc_copy_var(int ncid_in, int varid, int ncid_out);
 #define ncvarcpy(ncid_in, varid, ncid_out) ncvarcopy((ncid_in), (varid), (ncid_out))
 #endif
 
-/* End _var */
+/* End _var */
 /* Begin {put,get}_var1 */
 
+EXTERNL int
+nc_put_var1(int ncid, int varid, const size_t *indexp, const void *value);
+
+EXTERNL int
+nc_get_var1(int ncid, int varid, const size_t *indexp, void *value);
+
 EXTERNL int
 nc_put_var1_text(int ncid, int varid, const size_t *indexp, const char *op);
 
@@ -475,6 +517,14 @@ nc_get_var1_double(int ncid, int varid, const size_t *indexp, double *ip);
 /* End {put,get}_var1 */
 /* Begin {put,get}_vara */
 
+EXTERNL int
+nc_put_vara(int ncid, int varid,
+	 const size_t *start, const size_t *count, const void *value);
+
+EXTERNL int
+nc_get_vara(int ncid, int varid,
+	 const size_t *start, const size_t *count, void *value);
+
 EXTERNL int
 nc_put_vara_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const char *op);
@@ -482,7 +532,7 @@ nc_put_vara_text(int ncid, int varid,
 EXTERNL int
 nc_get_vara_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, char *ip);
-
+
 EXTERNL int
 nc_put_vara_uchar(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const unsigned char *op);
@@ -539,17 +589,19 @@ EXTERNL int
 nc_get_vara_double(int ncid, int varid,
 	const size_t *startp, const size_t *countp, double *ip);
 
-EXTERNL int
-nc_put_vara(int cdfid, int varid,
-	    const size_t *start, const size_t *edges, const void *value);
-
-EXTERNL int
-nc_get_vara(int cdfid, int varid,
-	    const size_t *start, const size_t *edges, void *value);
-
-/* End {put,get}_vara */
+/* End {put,get}_vara */
 /* Begin {put,get}_vars */
 
+EXTERNL int
+nc_put_vars(int ncid, int varid,
+	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
+	 const void * value);
+
+EXTERNL int
+nc_get_vars(int ncid, int varid,
+	 const size_t *start, const size_t *count, const ptrdiff_t *stride,
+	 void * value);
+
 EXTERNL int
 nc_put_vars_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -609,7 +661,7 @@ EXTERNL int
 nc_get_vars_long(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
 	long *ip);
-
+
 EXTERNL int
 nc_put_vars_float(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -633,6 +685,15 @@ nc_get_vars_double(int ncid, int varid,
 /* End {put,get}_vars */
 /* Begin {put,get}_varm */
 
+EXTERNL int
+nc_put_varm(int ncid, int varid, const size_t *start, const size_t *count, 
+            const ptrdiff_t *stride, const ptrdiff_t *imapp, 
+	    const void *value);
+
+EXTERNL int
+nc_get_varm(int ncid, int varid, const size_t *start, const size_t *count, 
+	    const ptrdiff_t *stride, const ptrdiff_t *imapp, void *value);
+
 EXTERNL int
 nc_put_varm_text(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -668,7 +729,7 @@ nc_get_varm_schar(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
 	const ptrdiff_t *imapp, 
 	signed char *ip);
-
+
 EXTERNL int
 nc_put_varm_short(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
@@ -726,7 +787,7 @@ nc_put_varm_double(int ncid, int varid,
 EXTERNL int
 nc_get_varm_double(int ncid, int varid,
 	const size_t *startp, const size_t *countp, const ptrdiff_t *stridep,
-	const ptrdiff_t * imap, 
+	const ptrdiff_t * imapp, 
 	double *ip);
 
 /* End {put,get}_varm */
@@ -780,6 +841,25 @@ nc_put_var_double(int ncid, int varid, const double *op);
 EXTERNL int
 nc_get_var_double(int ncid, int varid, double *ip);
 
+#ifdef LOGGING
+
+#ifdef DEBUG
+EXTERNL void
+nc_exit(void);
+#endif
+
+EXTERNL void 
+nc_set_log_level(int new_level);
+/* Use this to turn off logging by calling
+   nc_log_level(NC_TURN_OFF_LOGGING) */
+#define NC_TURN_OFF_LOGGING (-1)
+
+#else /* not LOGGING */
+
+#define nc_set_log_level(e)
+
+#endif
+
 /* End {put,get}_var */
 
 /* #ifdef _CRAYMPP */
@@ -805,6 +885,9 @@ EXTERNL int
 nc__open_mp(const char *path, int mode, int basepe,
 	size_t *chunksizehintp, int *ncidp);
 
+EXTERNL int
+nc_delete(const char * path);
+
 EXTERNL int
 nc_delete_mp(const char * path, int basepe);
 
@@ -816,7 +899,6 @@ nc_inq_base_pe(int ncid, int *pe);
 
 /* #endif _CRAYMPP */
 
-
 /* Begin v2.4 backward compatiblity */
 /*
  * defining NO_NETCDF_2 to the preprocessor
@@ -883,7 +965,7 @@ nc_advise(const char *cdf_routine_name, int err, const char *fmt,...);
  * This is the only thing in this file which architecture dependent.
  */
 typedef int nclong;
-
+
 EXTERNL int
 nctypelen(nc_type datatype);
 
@@ -925,7 +1007,7 @@ ncdiminq(int ncid, int dimid, char *name, long *lenp);
 
 EXTERNL int
 ncdimrename(int ncid, int dimid, const char *name);
-
+
 EXTERNL int
 ncattput(int ncid, int varid, const char *name, nc_type xtype,
 	int len, const void *op);
@@ -973,7 +1055,7 @@ ncvarput(int ncid, int varid, const long *startp, const long *countp,
 EXTERNL int
 ncvarget(int ncid, int varid, const long *startp, const long *countp, 
 	void *ip);
-
+
 EXTERNL int
 ncvarputs(int ncid, int varid, const long *startp, const long *countp,
 	const long *stridep, const void *op);
