@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: test_read.c 11906 2005-08-08 19:51:43Z root $
+ *   $Id: test_read.c 15015 2006-11-03 17:39:01Z jimg $
  *********************************************************************/
 
 #include "tests.h"
@@ -17,40 +17,42 @@ test_nc_strerror(void)
     int i;
     const char *message;
 
-    static struct {
+    static const struct {
 	int status;
-	char *msg;
+	const char *msg;
     } ncerrs[] = {
 	{NC_NOERR, "No error"},
-	{NC_EBADID, "Not a netCDF id"},
-	{NC_ENFILE, "Too many netCDF files open"},
-	{NC_EEXIST, "netCDF file exists && NC_NOCLOBBER"},
-	{NC_EINVAL, "Invalid argument"},
-	{NC_EPERM, "Write to read only"},
-	{NC_ENOTINDEFINE, "Operation not allowed in data mode"},
-	{NC_EINDEFINE, "Operation not allowed in define mode"},
-	{NC_EINVALCOORDS, "Index exceeds dimension bound"},
-	{NC_EMAXDIMS, "NC_MAX_DIMS exceeded"},
-	{NC_ENAMEINUSE, "String match to name in use"},
-	{NC_ENOTATT, "Attribute not found"},
-	{NC_EMAXATTS, "NC_MAX_ATTRS exceeded"},
-	{NC_EBADTYPE, "Not a netCDF data type or _FillValue type mismatch"},
-	{NC_EBADDIM, "Invalid dimension id or name"},
-	{NC_EUNLIMPOS, "NC_UNLIMITED in the wrong index"},
-	{NC_EMAXVARS, "NC_MAX_VARS exceeded"},
-	{NC_ENOTVAR, "Variable not found"},
-	{NC_EGLOBAL, "Action prohibited on NC_GLOBAL varid"},
-	{NC_ENOTNC, "Not a netCDF file"},
-	{NC_ESTS, "In Fortran, string too short"},
-	{NC_EMAXNAME, "NC_MAX_NAME exceeded"},
-	{NC_EUNLIMIT, "NC_UNLIMITED size already in use"},
-	{NC_ENORECVARS, "nc_rec op when there are no record vars"},
-	{NC_ECHAR, "Attempt to convert between text & numbers"},
-	{NC_EEDGE, "Edge+start exceeds dimension bound"},
-	{NC_ESTRIDE, "Illegal stride"},
-	{NC_EBADNAME, "Attribute or variable name contains illegal characters"},
-	{NC_ERANGE, "Numeric conversion not representable"},
-	{NC_ENOMEM, "Memory allocation (malloc) failure"},
+	{NC_EBADID, "NetCDF: Not a valid ID"},
+	{NC_ENFILE, "NetCDF: Too many files open"},
+	{NC_EEXIST, "NetCDF: File exists && NC_NOCLOBBER"},
+	{NC_EINVAL, "NetCDF: Invalid argument"},
+	{NC_EPERM, "NetCDF: Write to read only"},
+	{NC_ENOTINDEFINE, "NetCDF: Operation not allowed in data mode"},
+	{NC_EINDEFINE, "NetCDF: Operation not allowed in define mode"},
+	{NC_EINVALCOORDS, "NetCDF: Index exceeds dimension bound"},
+	{NC_EMAXDIMS, "NetCDF: NC_MAX_DIMS exceeded"},
+	{NC_ENAMEINUSE, "NetCDF: String match to name in use"},
+	{NC_ENOTATT, "NetCDF: Attribute not found"},
+	{NC_EMAXATTS, "NetCDF: NC_MAX_ATTRS exceeded"},
+	{NC_EBADTYPE, "NetCDF: Not a valid data type or _FillValue type mismatch"},
+	{NC_EBADDIM, "NetCDF: Invalid dimension ID or name"},
+	{NC_EUNLIMPOS, "NetCDF: NC_UNLIMITED in the wrong index"},
+	{NC_EMAXVARS, "NetCDF: NC_MAX_VARS exceeded"},
+	{NC_ENOTVAR, "NetCDF: Variable not found"},
+	{NC_EGLOBAL, "NetCDF: Action prohibited on NC_GLOBAL varid"},
+	{NC_ENOTNC, "NetCDF: Unknown file format"},
+	{NC_ESTS, "NetCDF: In Fortran, string too short"},
+	{NC_EMAXNAME, "NetCDF: NC_MAX_NAME exceeded"},
+	{NC_EUNLIMIT, "NetCDF: NC_UNLIMITED size already in use"},
+	{NC_ENORECVARS, "NetCDF: nc_rec op when there are no record vars"},
+	{NC_ECHAR, "NetCDF: Attempt to convert between text & numbers"},
+	{NC_EEDGE, "NetCDF: Start+count exceeds dimension bound"},
+	{NC_ESTRIDE, "NetCDF: Illegal stride"},
+	{NC_EBADNAME, "NetCDF: Name contains illegal characters"},
+	{NC_ERANGE, "NetCDF: Numeric conversion not representable"},
+	{NC_ENOMEM, "NetCDF: Memory allocation (malloc) failure"},
+	{NC_EVARSIZE, "NetCDF: One or more variable sizes violate format constraints"},
+	{NC_EDIMSIZE, "NetCDF: Invalid dimension size"}
     };
 
     /* Try on a bad error status */
@@ -95,7 +97,7 @@ test_nc_open(void)
 	error("nc_open of nonexistent file should have returned system error");
 
     /* Open a file that is not a netCDF file. */
-    err = nc_open("tests.h", NC_NOWRITE, &ncid);/* should fail */
+    err = nc_open("nc_test.o", NC_NOWRITE, &ncid);/* should fail */
     IF (err != NC_ENOTNC)
 	error("nc_open of non-netCDF file: status = %d", err);
 
@@ -116,21 +118,19 @@ test_nc_open(void)
     IF (ncid2 == ncid)
 	error("netCDF IDs for first and second nc_open calls should differ");
 
-    if (! read_only) {		/* tests using netCDF scratch file */
-	err = nc_create(scratch, NC_NOCLOBBER, &ncid2);
-	IF (err) 
-	    error("nc_create: %s", nc_strerror(err));
-	else 
-	    (void) nc_close(ncid2);
-	err = nc_open(scratch, NC_WRITE, &ncid2);
-	IF (err) 
-	    error("nc_open: %s", nc_strerror(err));
-	else 
-	    (void) nc_close(ncid2);
-	err = remove(scratch);
-	IF (err) 
-	    error("remove of %s failed", scratch);
-    }
+    err = nc_create(scratch, NC_NOCLOBBER, &ncid2);
+    IF (err) 
+       error("nc_create: %s", nc_strerror(err));
+    else 
+       (void) nc_close(ncid2);
+    err = nc_open(scratch, NC_WRITE, &ncid2);
+    IF (err) 
+       error("nc_open: %s", nc_strerror(err));
+    else 
+       (void) nc_close(ncid2);
+    err = remove(scratch);
+    IF (err) 
+       error("remove of %s failed", scratch);
 
     err = nc_close(ncid);
     IF (err)
@@ -174,17 +174,15 @@ test_nc_close(void)
     IF (err)
 	error("nc_close in data mode failed: %s", nc_strerror(err));
 
-    if (! read_only) {		/* tests using netCDF scratch file */
-        err = nc_create(scratch, NC_NOCLOBBER, &ncid);
-        IF (err) 
-            error("nc_create: %s", nc_strerror(err));
-	err = nc_close(ncid);
-	IF (err)
-	    error("nc_close in define mode: %s", nc_strerror(err));
-        err = remove(scratch);
-        IF (err)
-            error("remove of %s failed", scratch);
-    }
+    err = nc_create(scratch, NC_NOCLOBBER, &ncid);
+    IF (err) 
+       error("nc_create: %s", nc_strerror(err));
+    err = nc_close(ncid);
+    IF (err)
+       error("nc_close in define mode: %s", nc_strerror(err));
+    err = remove(scratch);
+    IF (err)
+       error("remove of %s failed", scratch);
 }
 
 
@@ -249,7 +247,7 @@ test_nc_inq(void)
     else IF (nvars != NVARS)
 	error("nc_inq subset: wrong number of variables returned, %d", nvars);
 
-    if (! read_only) {		/* tests using netCDF scratch file */
+    {		/* tests using netCDF scratch file */
 	int ncid2;		/* for scratch netCDF dataset */
 
         err = nc_create(scratch, NC_NOCLOBBER, &ncid2);
@@ -815,7 +813,6 @@ test_nc_inq_vartype(void)
 }
 
 
-#ifdef TEST_VOIDSTAR
 /*
  * Test nc_put_var1
  */
@@ -880,7 +877,6 @@ test_nc_get_var1(void)
     print_nok(nok);
 }
 
-
 /*
  * Test nc_get_vara
  * Choose a random point dividing each dim into 2 parts
@@ -1394,7 +1390,6 @@ test_nc_get_att(void)
 	error("nc_close: %s", nc_strerror(err));
     print_nok(nok);
 }
-#endif /* TEST_VOIDSTAR */
 
 
 void
