@@ -1,9 +1,10 @@
 /*********************************************************************
  *   Copyright 1996, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Id: tests.h 11906 2005-08-08 19:51:43Z root $
+ *   $Id: tests.h 15015 2006-11-03 17:39:01Z jimg $
  *********************************************************************/
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -123,7 +124,8 @@
 #define BAD_ATTNUM -1           /* invalid att number */
 #define BAD_TYPE (nc_type) 0    /* invalid data type */
 #define BAD_FILLMODE -1         /* invalid fill mode */
-#define BAD_NAME "a+b"		/* invalid name */
+#define BAD_NAME "a/b"		/* invalid name */
+#define BAD_DEFAULT_FORMAT 12   /* invalid default format */
 
 #define LEN_OF(array) ((sizeof array) / (sizeof array[0]))
 
@@ -152,6 +154,7 @@ extern char scratch[];		/* netCDF test file for writing */
 extern int  read_only;		/* if 1, don't try to change files */
 extern int  verbose;		/* if 1, print details of tests */
 extern int  nfails;		/* number of failures in specific test */
+extern int  max_nmpt;		/* max. number of messages per test */
 
     /* Global variables - test data */
 
@@ -268,6 +271,7 @@ extern void test_nc_get_att_short(void);
 extern void test_nc_get_att_text(void);
 extern void test_nc_get_att_uchar(void);
 
+extern void test_nc_put_att(void);
 extern void test_nc_put_var_double(void);
 extern void test_nc_put_var_float(void);
 extern void test_nc_put_var_int(void);
@@ -318,7 +322,6 @@ extern void test_nc_put_varm_text(void);
 extern void test_nc_put_varm_uchar(void);
 extern void test_nc_put_varm(void);
 
-extern void test_nc_put_att(void);
 extern void test_nc_put_att_double(void);
 extern void test_nc_put_att_float(void);
 extern void test_nc_put_att_int(void);
@@ -348,6 +351,7 @@ extern void test_nc_inq_atttype(void);
 extern void test_nc_rename_att(void);
 extern void test_nc_del_att(void);
 extern void test_nc_set_fill(void);
+extern void test_nc_set_default_format(void);
 
 void print_nok(int nok);
 
@@ -433,13 +437,6 @@ void check_file(char *filename);
 extern int
 nctypelen(nc_type datatype);
 
-extern int
-nc_put_att(int ncid, int varid, const char *name, nc_type datatype,
-	size_t len, const void *value);
-
-extern int
-nc_get_att(int ncid, int varid, const char *name, void *value);
-
 extern int
 nc_put_var1(int ncid, int varid, const size_t *index, const void *value);
 
