@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, University Corporation for Atmospheric Research
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /usr/local/cvs-repository/libnc-dap/ncdump/vardata.h,v 1.2 2004/02/25 00:52:54 jimg Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.h,v 1.4 2005/07/22 23:01:39 russ Exp $
  *********************************************************************/
 
 extern char *progname;		/* for error messages */
@@ -15,11 +15,19 @@ extern "C" {
 #endif
 
 /* Output the data for a single variable, in CDL syntax. */
-extern int vardata ( const struct ncvar*, /* variable */
+extern int vardata ( const ncvar_t*, /* variable */
 		     size_t [], /* variable dimension lengths */
 		     int, /* netcdf id */
 		     int, /* variable id */
-		     const struct fspec* /* formatting specs */
+		     const fspec_t* /* formatting specs */
+    );
+
+/* Output the data for a single variable, in NcML syntax. */
+extern int vardatax ( const ncvar_t*, /* variable */
+		     size_t [], /* variable dimension lengths */
+		     int, /* netcdf id */
+		     int, /* variable id */
+		     const fspec_t* /* formatting specs */
     );
 
 #ifdef __cplusplus
